import axios, { type AxiosResponse } from 'axios'

/**
 * Handles Axios requests and their potential errors.
 *
 * @template T - The expected type of the response data.
 * @param {Promise<AxiosResponse<T>>} request - The Axios request to be handled.
 * @returns {Promise<readonly [T, null] | readonly [null, Error]>} A promise that resolves to a tuple where:
 *   - Either [T, null] if the request was successful, where T is the response data.
 *   - Or [null, Error] if an error occurred.
 */
export const handleAxiosError = async <T>(
  request: Promise<AxiosResponse<T>>
): Promise<readonly [T, null] | readonly [null, Error]> => {
  try {
    const { data } = await request
    return [data, null] as const
  } catch (error) {
    if (axios.isAxiosError(error)) {
      return [
        null,
        new Error(error.response?.data?.message || error.message)
      ] as const
    }

    return [
      null,
      error instanceof Error ? error : new Error('An unknown error occurred')
    ] as const
  }
}
