import { consoleApiAxios } from '../config'
import { handleAxiosError } from '../errorHandler'
import { sortApplicationsByCreationDate } from '../helper'
import type { AxiosRequestConfig } from 'axios'
import type {
  AddApplicationParams,
  GetApplicationParams,
  ApplicationType,
  ApplicationDetailType,
  CreateApiKeyResponse,
  DowngradeApplicationPlanBody,
  AddApplicationResponse,
  GetApplicationQuotaHistoryParams,
  QuotaHistoryResponse,
  RenewApplicationBody
} from './application.type'

/**
 * Adds a new application to the system.
 *
 * @param accessToken - The access token for authentication.
 * @param params - The parameters for creating the new application.
 * @param params.name - The name of the new application.
 * @param params.tags - The tags associated with the new application.
 * @param params.planId - The ID of the plan for the new application.
 * @param params.redirectUrl - The redirect URL for the new application.
 * @returns A promise that resolves to a tuple where the first element is the AddApplicationResponse if successful,
 *          and the second element is an Error if an error occurred.
 */
export const addApplication = async (
  accessToken: string,
  params: AddApplicationParams
) => {
  const config: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  return handleAxiosError(
    consoleApiAxios.post<AddApplicationResponse>(
      `/admin/app/`,
      {
        name: params.name,
        tags: params.tags,
        plan_id: params.planId,
        redirect_url: params.redirectUrl
      },
      config
    )
  )
}

/**
 * Retrieves a list of applications based on the provided parameters.
 *
 * @param accessToken - The access token for authentication.
 * @param params - The parameters for fetching applications.
 * @param params.productId - The ID of the product to filter applications.
 * @param params.page - The page number for pagination.
 * @param params.perPage - The number of items per page.
 * @returns A promise that resolves to a tuple where the first element is an array of ApplicationType if successful,
 *          sorted by creation date in descending order, and the second element is an Error if an error occurred.
 */
export const getApplication = async (
  accessToken: string,
  params: GetApplicationParams
) => {
  const config: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  const [data, error] = await handleAxiosError(
    consoleApiAxios.get<ApplicationType[]>(`/admin/app/`, {
      ...config,
      params: {
        product_id: params.productId,
        page: params.page,
        per_page: params.perPage
      }
    })
  )

  if (error) {
    return [null, error]
  }

  const sortedData = sortApplicationsByCreationDate(data)

  return [sortedData, null]
}

/**
 * Retrieves detailed information about a specific application.
 *
 * @param accessToken - The access token for authentication.
 * @param appId - The unique identifier of the application to retrieve details for.
 * @returns A promise that resolves to a tuple where the first element is the ApplicationDetailType if successful,
 *          and the second element is an Error if an error occurred. If successful, the Error will be null,
 *          and if an error occurs, the ApplicationDetailType will be null.
 */
export const getDetailApplication = async (
  accessToken: string,
  appId: string
) => {
  const config: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  return handleAxiosError(
    consoleApiAxios.get<ApplicationDetailType>(`/admin/app/${appId}/`, config)
  )
}

/**
 * Adds a new API key for a specific application.
 *
 * @param accessToken - The access token for authentication.
 * @param appId - The unique identifier of the application to add the API key to.
 * @returns A promise that resolves to a tuple where the first element is the CreateApiKeyResponse if successful,
 *          and the second element is an Error if an error occurred.
 */
export const addApiKey = async (accessToken: string, appId: number) => {
  const config: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  return handleAxiosError(
    consoleApiAxios.post<CreateApiKeyResponse>(
      `/admin/app/${appId}/api_key/`,
      {
        label: `Key ${Date.now().toString()}`
      },
      config
    )
  )
}

/**
 * Downgrades the plan of a specific application.
 *
 * @param accessToken - The access token for authentication.
 * @param appId - The unique identifier of the application to downgrade.
 * @param body - The request body containing the downgrade plan details.
 * @returns A promise that resolves to a tuple where the first element is the response data if successful,
 *          and the second element is an Error if an error occurred.
 */
export const downgradeApplicationPlan = async (
  accessToken: string,
  appId: string,
  body: DowngradeApplicationPlanBody
) => {
  const config: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  return handleAxiosError<unknown>(
    consoleApiAxios.post(`/admin/app/${appId}/downgrade-plan/`, body, config)
  )
}

/**
 * Retrieves the quota history for a specific application.
 *
 * @param accessToken - The access token for authentication.
 * @param appId - The unique identifier of the application.
 * @param body - The request body containing any necessary parameters for the quota history.
 * @returns A promise that resolves to a tuple where the first element is the quota history data if successful,
 *          and the second element is an Error if an error occurred.
 */
export const getApplicationQuotaHistory = async (
  accessToken: string,
  appId: string,
  body: GetApplicationQuotaHistoryParams
) => {
  const requestConfig: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  return handleAxiosError(
    consoleApiAxios.post<QuotaHistoryResponse>(
      `/admin/app/${appId}/quota-history`,
      body,
      requestConfig
    )
  )
}

/**
 * Renews an application with the specified ID.
 *
 * @param accessToken - The access token for authentication.
 * @param id - The unique identifier of the application to renew.
 * @param body - The request body containing renewal details.
 * @returns A promise that resolves to a tuple where the first element is the response data if successful,
 *          and the second element is an Error if an error occurred.
 */
export const renewApplication = async (
  accessToken: string,
  id: string,
  body: RenewApplicationBody
) => {
  const requestConfig: AxiosRequestConfig = {
    headers: {
      Authorization: `Bearer ${accessToken}`
    }
  }

  return handleAxiosError(
    consoleApiAxios.post<unknown>(
      `/admin/app/${id}/renew/`,
      body,
      requestConfig
    )
  )
}
